@@ -1,10 +1,8 @@
 #!/usr/bin/env python
-# -*- coding: utf-8 -*-
 
-from __future__ import unicode_literals
 
+import ipaddress
 import sys
-from typing import Dict
 import unittest
 
 sys.path.append("..")
@@ -13,6 +11,9 @@ import geoip2.models
 
 
 class TestModels(unittest.TestCase):
+    def setUp(self) -> None:
+        self.maxDiff = 20_000
+
     def test_insights_full(self) -> None:
         raw = {
             "city": {
@@ -72,18 +73,19 @@ class TestModels(unittest.TestCase):
             "traits": {
                 "autonomous_system_number": 1234,
                 "autonomous_system_organization": "AS Organization",
+                "connection_type": "Cable/DSL",
                 "domain": "example.com",
                 "ip_address": "1.2.3.4",
                 "is_anonymous": True,
                 "is_anonymous_proxy": True,
                 "is_anonymous_vpn": True,
+                "is_anycast": True,
                 "is_hosting_provider": True,
                 "is_public_proxy": True,
                 "is_residential_proxy": True,
                 "is_satellite_provider": True,
                 "is_tor_exit_node": True,
                 "isp": "Comcast",
-                "network_speed": "cable/DSL",
                 "organization": "Blorg",
                 "static_ip_score": 1.3,
                 "user_count": 2,
@@ -91,12 +93,16 @@ class TestModels(unittest.TestCase):
             },
         }
 
-        model = geoip2.models.Insights(raw)
+        model = geoip2.models.Insights(["en"], **raw)  # type: ignore
         self.assertEqual(
-            type(model), geoip2.models.Insights, "geoip2.models.Insights object"
+            type(model),
+            geoip2.models.Insights,
+            "geoip2.models.Insights object",
         )
         self.assertEqual(
-            type(model.city), geoip2.records.City, "geoip2.records.City object"
+            type(model.city),
+            geoip2.records.City,
+            "geoip2.records.City object",
         )
         self.assertEqual(
             type(model.continent),
@@ -104,7 +110,9 @@ class TestModels(unittest.TestCase):
             "geoip2.records.Continent object",
         )
         self.assertEqual(
-            type(model.country), geoip2.records.Country, "geoip2.records.Country object"
+            type(model.country),
+            geoip2.records.Country,
+            "geoip2.records.Country object",
         )
         self.assertEqual(
             type(model.registered_country),
@@ -127,23 +135,35 @@ class TestModels(unittest.TestCase):
             "geoip2.records.Subdivision object",
         )
         self.assertEqual(
-            type(model.traits), geoip2.records.Traits, "geoip2.records.Traits object"
+            type(model.traits),
+            geoip2.records.Traits,
+            "geoip2.records.Traits object",
         )
-        self.assertEqual(model.raw, raw, "raw method returns raw input")
+        self.assertEqual(model.to_dict(), raw, "to_dict() method matches raw input")
         self.assertEqual(
-            model.subdivisions[0].iso_code, "MN", "div 1 has correct iso_code"
+            model.subdivisions[0].iso_code,
+            "MN",
+            "div 1 has correct iso_code",
         )
         self.assertEqual(
-            model.subdivisions[0].confidence, 88, "div 1 has correct confidence"
+            model.subdivisions[0].confidence,
+            88,
+            "div 1 has correct confidence",
         )
         self.assertEqual(
-            model.subdivisions[0].geoname_id, 574635, "div 1 has correct geoname_id"
+            model.subdivisions[0].geoname_id,
+            574635,
+            "div 1 has correct geoname_id",
         )
         self.assertEqual(
-            model.subdivisions[0].names, {"en": "Minnesota"}, "div 1 names are correct"
+            model.subdivisions[0].names,
+            {"en": "Minnesota"},
+            "div 1 names are correct",
         )
         self.assertEqual(
-            model.subdivisions[1].name, "Hennepin", "div 2 has correct name"
+            model.subdivisions[1].name,
+            "Hennepin",
+            "div 2 has correct name",
         )
         self.assertEqual(
             model.subdivisions.most_specific.iso_code,
@@ -165,12 +185,14 @@ class TestModels(unittest.TestCase):
         self.assertEqual(model.location.longitude, 93.2636, "correct longitude")
         self.assertEqual(model.location.metro_code, 765, "correct metro_code")
         self.assertEqual(
-            model.location.population_density, 1341, "correct population_density"
+            model.location.population_density,
+            1341,
+            "correct population_density",
         )
 
         self.assertRegex(
             str(model),
-            r"^geoip2.models.Insights\(\{.*geoname_id.*\}, \[.*en.*\]\)",
+            r"^geoip2.models.Insights\(\[.*en.*\]\, .*geoname_id.*\)",
             "Insights str representation looks reasonable",
         )
 
@@ -183,7 +205,9 @@ class TestModels(unittest.TestCase):
         )
 
         self.assertEqual(
-            model.location, eval(repr(model.location)), "Location repr can be eval'd"
+            model.location,
+            eval(repr(model.location)),
+            "Location repr can be eval'd",
         )
 
         self.assertIs(model.country.is_in_european_union, False)
@@ -193,6 +217,7 @@ class TestModels(unittest.TestCase):
         self.assertIs(model.traits.is_anonymous, True)
         self.assertIs(model.traits.is_anonymous_proxy, True)
         self.assertIs(model.traits.is_anonymous_vpn, True)
+        self.assertIs(model.traits.is_anycast, True)
         self.assertIs(model.traits.is_hosting_provider, True)
         self.assertIs(model.traits.is_public_proxy, True)
         self.assertIs(model.traits.is_residential_proxy, True)
@@ -202,12 +227,16 @@ class TestModels(unittest.TestCase):
         self.assertEqual(model.traits.static_ip_score, 1.3)
 
     def test_insights_min(self) -> None:
-        model = geoip2.models.Insights({"traits": {"ip_address": "5.6.7.8"}})
+        model = geoip2.models.Insights(["en"], traits={"ip_address": "5.6.7.8"})
         self.assertEqual(
-            type(model), geoip2.models.Insights, "geoip2.models.Insights object"
+            type(model),
+            geoip2.models.Insights,
+            "geoip2.models.Insights object",
         )
         self.assertEqual(
-            type(model.city), geoip2.records.City, "geoip2.records.City object"
+            type(model.city),
+            geoip2.records.City,
+            "geoip2.records.City object",
         )
         self.assertEqual(
             type(model.continent),
@@ -215,7 +244,9 @@ class TestModels(unittest.TestCase):
             "geoip2.records.Continent object",
         )
         self.assertEqual(
-            type(model.country), geoip2.records.Country, "geoip2.records.Country object"
+            type(model.country),
+            geoip2.records.Country,
+            "geoip2.records.Country object",
         )
         self.assertEqual(
             type(model.registered_country),
@@ -228,16 +259,19 @@ class TestModels(unittest.TestCase):
             "geoip2.records.Location object",
         )
         self.assertEqual(
-            type(model.traits), geoip2.records.Traits, "geoip2.records.Traits object"
+            type(model.traits),
+            geoip2.records.Traits,
+            "geoip2.records.Traits object",
         )
         self.assertEqual(
             type(model.subdivisions.most_specific),
             geoip2.records.Subdivision,
-            "geoip2.records.Subdivision object returned even"
-            "when none are available.",
+            "geoip2.records.Subdivision object returned even when none are available.",
         )
         self.assertEqual(
-            model.subdivisions.most_specific.names, {}, "Empty names hash returned"
+            model.subdivisions.most_specific.names,
+            {},
+            "Empty names hash returned",
         )
 
     def test_city_full(self) -> None:
@@ -262,10 +296,12 @@ class TestModels(unittest.TestCase):
                 "is_satellite_provider": True,
             },
         }
-        model = geoip2.models.City(raw)
+        model = geoip2.models.City(["en"], **raw)  # type: ignore
         self.assertEqual(type(model), geoip2.models.City, "geoip2.models.City object")
         self.assertEqual(
-            type(model.city), geoip2.records.City, "geoip2.records.City object"
+            type(model.city),
+            geoip2.records.City,
+            "geoip2.records.City object",
         )
         self.assertEqual(
             type(model.continent),
@@ -273,7 +309,9 @@ class TestModels(unittest.TestCase):
             "geoip2.records.Continent object",
         )
         self.assertEqual(
-            type(model.country), geoip2.records.Country, "geoip2.records.Country object"
+            type(model.country),
+            geoip2.records.Country,
+            "geoip2.records.Country object",
         )
         self.assertEqual(
             type(model.registered_country),
@@ -286,16 +324,26 @@ class TestModels(unittest.TestCase):
             "geoip2.records.Location object",
         )
         self.assertEqual(
-            type(model.traits), geoip2.records.Traits, "geoip2.records.Traits object"
+            type(model.traits),
+            geoip2.records.Traits,
+            "geoip2.records.Traits object",
+        )
+        self.assertEqual(
+            model.to_dict(),
+            raw,
+            "to_dict method output matches raw input",
         )
-        self.assertEqual(model.raw, raw, "raw method returns raw input")
         self.assertEqual(model.continent.geoname_id, 42, "continent geoname_id is 42")
         self.assertEqual(model.continent.code, "NA", "continent code is NA")
         self.assertEqual(
-            model.continent.names, {"en": "North America"}, "continent names is correct"
+            model.continent.names,
+            {"en": "North America"},
+            "continent names is correct",
         )
         self.assertEqual(
-            model.continent.name, "North America", "continent name is correct"
+            model.continent.name,
+            "North America",
+            "continent name is correct",
         )
         self.assertEqual(model.country.geoname_id, 1, "country geoname_id is 1")
         self.assertEqual(model.country.iso_code, "US", "country iso_code is US")
@@ -305,11 +353,15 @@ class TestModels(unittest.TestCase):
             "country names is correct",
         )
         self.assertEqual(
-            model.country.name, "United States of America", "country name is correct"
+            model.country.name,
+            "United States of America",
+            "country name is correct",
         )
         self.assertEqual(model.country.confidence, None, "country confidence is None")
         self.assertEqual(
-            model.registered_country.iso_code, "CA", "registered_country iso_code is CA"
+            model.registered_country.iso_code,
+            "CA",
+            "registered_country iso_code is CA",
         )
         self.assertEqual(
             model.registered_country.names,
@@ -326,67 +378,75 @@ class TestModels(unittest.TestCase):
             False,
             "traits is_anonymous_proxy returns False by default",
         )
+        self.assertEqual(
+            model.traits.is_anycast,
+            False,
+            "traits is_anycast returns False by default",
+        )
         self.assertEqual(
             model.traits.is_satellite_provider,
             True,
             "traits is_setellite_provider is True",
         )
-        self.assertEqual(model.raw, raw, "raw method produces raw output")
+        self.assertEqual(model.to_dict(), raw, "to_dict method matches raw input")
 
         self.assertRegex(
-            str(model), r"^geoip2.models.City\(\{.*geoname_id.*\}, \[.*en.*\]\)"
+            str(model),
+            r"^geoip2.models.City\(\[.*en.*\], .*geoname_id.*\)",
         )
 
         self.assertFalse(model == True, "__eq__ does not blow up on weird input")
 
     def test_unknown_keys(self) -> None:
         model = geoip2.models.City(
-            {
-                "city": {"invalid": 0},
-                "continent": {
-                    "invalid": 0,
-                    "names": {"invalid": 0},
-                },
-                "country": {
-                    "invalid": 0,
-                    "names": {"invalid": 0},
-                },
-                "location": {"invalid": 0},
-                "postal": {"invalid": 0},
-                "subdivisions": [
-                    {
-                        "invalid": 0,
-                        "names": {
-                            "invalid": 0,
-                        },
-                    },
-                ],
-                "registered_country": {
+            ["en"],
+            city={"invalid": 0},
+            continent={
+                "invalid": 0,
+                "names": {"invalid": 0},
+            },
+            country={
+                "invalid": 0,
+                "names": {"invalid": 0},
+            },
+            location={"invalid": 0},
+            postal={"invalid": 0},
+            subdivisions=[
+                {
                     "invalid": 0,
                     "names": {
                         "invalid": 0,
                     },
                 },
-                "represented_country": {
+            ],
+            registered_country={
+                "invalid": 0,
+                "names": {
                     "invalid": 0,
-                    "names": {
-                        "invalid": 0,
-                    },
                 },
-                "traits": {"ip_address": "1.2.3.4", "invalid": "blah"},
-                "unk_base": {"blah": 1},
-            }
+            },
+            represented_country={
+                "invalid": 0,
+                "names": {
+                    "invalid": 0,
+                },
+            },
+            traits={"ip_address": "1.2.3.4", "invalid": "blah"},
+            unk_base={"blah": 1},
         )
         with self.assertRaises(AttributeError):
             model.unk_base  # type: ignore
         with self.assertRaises(AttributeError):
             model.traits.invalid  # type: ignore
-        self.assertEqual(model.traits.ip_address, "1.2.3.4", "correct ip")
+        self.assertEqual(
+            model.traits.ip_address,
+            ipaddress.ip_address("1.2.3.4"),
+            "correct ip",
+        )
 
 
 class TestNames(unittest.TestCase):
-
-    raw: Dict = {
+    raw: dict = {
         "continent": {
             "code": "NA",
             "geoname_id": 42,
@@ -415,7 +475,7 @@ class TestNames(unittest.TestCase):
     }
 
     def test_names(self) -> None:
-        model = geoip2.models.Country(self.raw, locales=["sq", "ar"])
+        model = geoip2.models.Country(["sq", "ar"], **self.raw)
         self.assertEqual(
             model.continent.names,
             self.raw["continent"]["names"],
@@ -428,7 +488,7 @@ class TestNames(unittest.TestCase):
         )
 
     def test_three_locales(self) -> None:
-        model = geoip2.models.Country(self.raw, locales=["fr", "zh-CN", "en"])
+        model = geoip2.models.Country(locales=["fr", "zh-CN", "en"], **self.raw)
         self.assertEqual(
             model.continent.name,
             "北美洲",
@@ -437,27 +497,33 @@ class TestNames(unittest.TestCase):
         self.assertEqual(model.country.name, "États-Unis", "country name is in French")
 
     def test_two_locales(self) -> None:
-        model = geoip2.models.Country(self.raw, locales=["ak", "fr"])
+        model = geoip2.models.Country(locales=["ak", "fr"], **self.raw)
         self.assertEqual(
             model.continent.name,
             None,
-            "continent name is undef (no Akan or French " "available)",
+            "continent name is undef (no Akan or French available)",
         )
         self.assertEqual(model.country.name, "États-Unis", "country name is in French")
 
     def test_unknown_locale(self) -> None:
-        model = geoip2.models.Country(self.raw, locales=["aa"])
+        model = geoip2.models.Country(locales=["aa"], **self.raw)
         self.assertEqual(
-            model.continent.name, None, "continent name is undef (no Afar available)"
+            model.continent.name,
+            None,
+            "continent name is undef (no Afar available)",
         )
         self.assertEqual(
-            model.country.name, None, "country name is in None (no Afar available)"
+            model.country.name,
+            None,
+            "country name is in None (no Afar available)",
         )
 
     def test_german(self) -> None:
-        model = geoip2.models.Country(self.raw, locales=["de"])
+        model = geoip2.models.Country(locales=["de"], **self.raw)
         self.assertEqual(
-            model.continent.name, "Nordamerika", "Correct german name for continent"
+            model.continent.name,
+            "Nordamerika",
+            "Correct german name for continent",
         )
 
 
